"""Play a level"""

import ground
import camera
from pyglet.gl import *
import mob
import reticule
import misc
import yaml
import enemy
from pyglet.window import key
import player
import fence
import math
import porch

class Playlevel(object):

    def __init__(self,g):
        self.g = g
        self.g.scene = self
        self.gamefield_size = 2048

        self.camera = camera.Camera()
        self.camera.xrot= -60.0
        self.camera.yrot= 0.0
        self.camera.xpos = 1024.0
        self.camera.ypos = -370.0
        self.camera.zoom = -500.0

        # how far behind george the camera should be, along game y axis
        self.cam_george_offset = 800.0
        # min camera y position
        self.cam_miny = -370.0

        # Mouse position
        self.mx, self.my = 0, 0

        # Ground
        self.ground = ground.Ground()

        # Porch
        self.porch = porch.Porch(self.g)

        # Reticule
        self.reticule = reticule.Reticule()

        self.moblist = []
        self.generate_fence()
        self.level_load()

        # George (player)
        self.george = player.Player(self.g, 1024.0, -50.0)
        self.moblist.append(self.george)

        # test enemy, change this
        testenemy = enemy.Enemy(self.g, self.enemy_hit, 1024.0, 1024.0,
                                'eggboy')
        self.moblist.append(testenemy)

        self.score = 0


    # Mouse and keyboard handlers

    def on_mouse_drag(self, x, y, dx, dy, buttons, mods):
        self.on_mouse_motion(x, y, dx, dy)

    def on_mouse_motion(self, x, y, dx, dy):
        self.mx, self.my = x, y

    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        pass

    def on_mouse_press(self, x, y, buttons, mods):
        # left click - move george to this position
        if buttons & pyglet.window.mouse.LEFT:
            world_x, world_y, world_z = misc.screen_to_world(x, y)
            self.george.porch_move(world_x)

    def on_key_press(self, keysym, mod):
        if keysym == key.SPACE:
            self.george.throw(self.reticule.x, self.reticule.y)


    # Draw the scene
    def on_draw(self):
        # Aim camera at george
        #self.camera.ypos = self.george.y - self.cam_george_offset
        #if self.camera.ypos < self.cam_miny:
        #    self.camera.ypos = self.cam_miny
        #degrad = math.atan2(self.george.y - self.camera.ypos,
        #                    self.george.x - self.camera.xpos)
        #self.camera.yrot = (math.pi / 2.0 - degrad) / (math.pi * 2.0) * 360.0

        self.camera.xpos = self.george.x

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        glRotatef(self.camera.xrot,1.0,0.0,0.0);
        glRotatef(self.camera.yrot,0.0,0.0,1.0);
        glTranslatef(-self.camera.xpos, -self.camera.ypos, self.camera.zoom)

        # Update reticule position for new camera
        self.reticule.update_position(self.mx, self.my)
        
        self.ground.draw()
        self.porch.draw()
        self.reticule.draw()

        # Draw fence first since side fences are a special case for ordering
        for mob in self.moblist:
            if mob.type == 'fence':
                mob.draw()
        for mob in sorted(self.moblist, lambda a,b: cmp(b.y, a.y)):
            if mob.type != 'fence':
                mob.draw()


    # Simulation step
    def run(self):
        for mob in self.moblist:
            mob.simstep()


    # Load level
    def level_load(self):
        configfile = open(misc.configpath('level_1.yaml'), 'r')
        moblist = yaml.load(configfile)
        configfile.close()
        for mobinfo in moblist:
            self.moblist.append(mob.Mob(self.g, mobinfo['x'], mobinfo['y'],
                                        mobinfo['type']))


    # Enemy death handler
    def enemy_hit(self, enemy):
        #self.moblist.remove(enemy)
        #self.score += 1
        #print "You nailed a %s!  New score is %d" % (enemy.type, self.score)
        self.g.gamestate = 'winscreen_init'
    
        
    # Add fence to mob list
    def generate_fence(self):
        fencewidth = self.g.imagedict['fence'].width
        gatewidth = self.g.imagedict['gate'].width
        x = fencewidth / 2.0
        y = 2048.0

        # back fence and gate
        for i in range(4):
            self.moblist.append(fence.Fence(self.g, x, y))
            x += fencewidth
        x = x - fencewidth / 2.0 + gatewidth / 2.0
        self.moblist.append(mob.Mob(self.g, x, y, 'gate'))
        x += gatewidth / 2.0 + fencewidth / 2.0
        for i in range(4):
            self.moblist.append(fence.Fence(self.g, x, y))
            x += fencewidth

        # side fences
        rightx = x - fencewidth / 2.0
        y -= fencewidth / 2.0
        for i in range(8):
            self.moblist.append(fence.Fence(self.g, 0, y, True))
            self.moblist.append(fence.Fence(self.g, rightx, y, True))
            y -= fencewidth

        
