"""Rendering and window setup"""

from pyglet.gl import *
import pyglet

def create_display(g):
    win = pyglet.window.Window(resizable=True, vsync = True, width = 800,
                               height = 600, caption = "Lawn Order: SVU")
    win.g = g
    win.g.nearclip = 0.1
    win.g.farclip = 10000.0
    # One-time GL setup
    glClearColor(0.53, 0.80, 0.98, 0)
    #    glEnable(GL_DEPTH_TEST)
    glDisable(GL_CULL_FACE)
    glShadeModel (GL_SMOOTH)
    glEnable(GL_TEXTURE_2D)
    #    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
    glLightfv(GL_LIGHT1, GL_AMBIENT,(GLfloat*4)(*(0.7,0.7,0.7,1.0)));
    glLightfv(GL_LIGHT1, GL_DIFFUSE,(GLfloat*4)(*(1.0,1.0,1.0,1.0)));
    glEnable(GL_LIGHT1)
    #    glEnable(GL_LIGHTING)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    @win.event
    def on_draw():
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_draw()

    @win.event
    def on_resize(width, height):
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(60.0, width / float(height),
                       win.g.nearclip, win.g.farclip)
        glMatrixMode(GL_MODELVIEW)
        return pyglet.event.EVENT_HANDLED

    @win.event
    def on_mouse_drag(x, y, dx, dy, buttons, mods):
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_mouse_drag(x, y, dx, dy, buttons, mods)

    @win.event
    def on_mouse_motion(x, y, dx, dy):
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_mouse_motion(x, y, dx, dy)

    @win.event
    def on_mouse_scroll(x, y, scroll_x, scroll_y):
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_mouse_scroll(x, y, scroll_x, scroll_y)

    @win.event
    def on_mouse_press(x, y, buttons, mods):
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_mouse_press(x, y, buttons, mods)

    @win.event
    def on_key_press(keysym, mod):
        if win.g.gamestate != 'run':
            return
        win.g.scene.on_key_press(keysym, mod)


    return win

